﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace MyClock
{
    public partial class About : Form
    {
        public About()
        {
            InitializeComponent();
        }

        /// <Ver 1.16.2>Aboutダイアログで起動時表示のフラグ
        public static bool m_show_about;

        /// <summary>
        /// <Ver 1.14.1></Ver>
        /// Aboutダイアログ描画イベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void About_Load(object sender, EventArgs e)
        {
            // <Ver 1.14.1>起動時にAboutダイアログを表示するチェックボックスの設定
            if (m_show_about)
                checkBox_about_show.CheckState = CheckState.Checked;
            else
                checkBox_about_show.CheckState = CheckState.Unchecked;

            // <Ver 1.14.2>バージョン情報表示 - アセンブリ情報を利用
            string msg = "";
            this.label_about_itle.Text = getAssemblyTitle();
            msg = getAssemblyProduct() + " Ver. " + getAssemblyVersion() + "\r\n";
            msg += getAssemblyCopyright() + "\r\n\r\n";
            textBox_about_prodctInfo.Text = msg;

            // <Ver 1.16.9>[F1]キーから表示した時のみ[外観の設定]ボタンを表示する
            if (Display.m_crtl_available)
                button_about_ctrl.Visible = true;
            else
                button_about_ctrl.Visible = false;

            // <Ver 1.16.9>[外観の設定]ボタン表示フラグをfalseに戻す
            Display.m_crtl_available = false;

            // <Ver 1.18.0>ログイン時にMyClockを起動のチェックボックスの設定
            // INIファイルの情報に従ってチェックボックスとstartupのショートカットを設定する
            if (Display.m_startup)
                checkBox_Startup.CheckState = CheckState.Checked;
            else
                checkBox_Startup.CheckState = CheckState.Unchecked;
            SetStartup();

            //<Ver 1.16.3>OKボタンにフォーカス
            button_about_OK.Select();

            // <Ver 1.16.3>Aboutダイアログを時計の周りに重ならないように表示する
            // Aboutダイアログがあるディスプレイの高さと幅を取得
            int screenH = System.Windows.Forms.Screen.GetBounds(this).Height;
            int screenW = System.Windows.Forms.Screen.GetBounds(this).Width;

            // MyClockフォームの左上の位置(left, top, width, height)を取得
            var left = this.Owner.Location.X;
            var top = this.Owner.Location.Y;
            var width = this.Owner.Width;
            var height = this.Owner.Height;

            // Aboutダイアログのデフォルトの位置(Dislayフォームの右隣)
            int X = left + width;
            int Y = top;
            // <Ver 1.16.3>Aboutダイアログがスクリーンからはみ出す時、位置(X, Y)を更新
            if (X + this.Width > screenW)
                X = left - this.Width;
            if (Y + this.Height > screenH)
                Y = top + height - this.Height;

            // 親ウィンドウ (MyClock)の周りに移動させる
            this.Location = new Point(X, Y);
        }

        /// <summary>
        /// <Ver 1.14.2></Ver>
        /// アセンブリ情報の「タイトル」を返す
        /// </summary>
        /// <returns>タイトルの文字列</returns>
        public static string getAssemblyTitle()
        {
            System.Reflection.AssemblyTitleAttribute title =
                (System.Reflection.AssemblyTitleAttribute)Attribute.GetCustomAttribute(
                    System.Reflection.Assembly.GetExecutingAssembly(),
                    typeof(System.Reflection.AssemblyTitleAttribute));

            return title.Title.ToString();
        }

        /// <summary>
        /// <Ver 1.14.2></Ver>
        /// アセンブリ情報の「製品」を返す
        /// </summary>
        /// <returns>製品の文字列</returns>
        public static string getAssemblyProduct()
        {
            System.Reflection.AssemblyProductAttribute product =
                (System.Reflection.AssemblyProductAttribute)Attribute.GetCustomAttribute(
                    System.Reflection.Assembly.GetExecutingAssembly(),
                    typeof(System.Reflection.AssemblyProductAttribute));

            return product.Product.ToString();
        }

        /// <summary>
        /// <Ver 1.14.2></Ver>
        /// アセンブリ情報の「バージョン」を返す
        /// </summary>
        /// <returns>バージョンの文字列</returns>
        public static string getAssemblyVersion()
        {
            //自分自身のAssembly情報を取得
            System.Reflection.Assembly assembly =
                System.Reflection.Assembly.GetExecutingAssembly();
            //バージョンの取得
            System.Version ver = assembly.GetName().Version;

            return ver.ToString();
        }

        /// <summary>
        /// <Ver 1.14.2></Ver>
        /// アセンブリ情報の「著作権」を返す
        /// </summary>
        /// <returns>著作権の文字列</returns>
        private string getAssemblyCopyright()
        {
            System.Reflection.AssemblyCopyrightAttribute copyright =
                (System.Reflection.AssemblyCopyrightAttribute)Attribute.GetCustomAttribute(
                    System.Reflection.Assembly.GetExecutingAssembly(),
                    typeof(System.Reflection.AssemblyCopyrightAttribute));

            return copyright.Copyright.ToString();
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// ログイン時にMyClockを起動するかどうかを設定する
        /// - チェックボックスの設定のみに従ってstartupのショートカットを追加/削除する
        /// </summary>
        private void SetStartup()
        {
            // スタートアップフォルダのパスにショートカットファイル名を結合する
            string shortcutPath = System.IO.Path.Combine(
                System.Environment.GetFolderPath(Environment.SpecialFolder.Startup),
                @"MyClock.lnk");
            // MyClock.exe のパスを取得する
            string myclockPath = Application.ExecutablePath;

            // WshShellを作成する
            IWshRuntimeLibrary.WshShell shell = new IWshRuntimeLibrary.WshShell();
            //ショートカットのパスを指定して、WshShortcutを作成
            IWshRuntimeLibrary.IWshShortcut shortcut =
                (IWshRuntimeLibrary.IWshShortcut)shell.CreateShortcut(shortcutPath);
            
            // リンク先を指定する
            shortcut.TargetPath = myclockPath;
            // 実行時の大きさ 1が通常、3が最大化、7が最小化
            shortcut.WindowStyle = 1;
            // コメント
            shortcut.Description = "MyClock";
            // アイコンへのパス - MyClock.exe のインデックス0のアイコン
            shortcut.IconLocation = Application.ExecutablePath + ",0";

            switch (checkBox_Startup.CheckState)
            {
                case CheckState.Checked:
                    // ショートカットを作成
                    shortcut.Save();
                    // フラグを立てる
                    Display.m_startup = true;
                    break;
                case CheckState.Indeterminate:
                case CheckState.Unchecked:
                    if (System.IO.File.Exists(@shortcutPath))
                    {
                        System.IO.File.Delete(@shortcutPath);
                        // フラグを下げる
                        Display.m_startup = false;
                    }
                    break;
            }
            // 後始末
            System.Runtime.InteropServices.Marshal.FinalReleaseComObject(shortcut);
            System.Runtime.InteropServices.Marshal.FinalReleaseComObject(shell);
        }

        /// <summary>
        /// <Ver 1.16.2></Ver>
        /// チェックボックスのイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkBox_about_show_CheckedChanged(object sender, EventArgs e)
        {
            switch (checkBox_about_show.CheckState)
            {
                case CheckState.Checked:
                    m_show_about = true;
                    break;
                case CheckState.Unchecked:
                case CheckState.Indeterminate:
                    m_show_about = false;
                    break;
            }
            button_about_OK.Select();
        }

        /// <summary>
        /// <Ver 1.16.9></Ver>
        /// [外観の変更]ボタンクリックのイベントハンドラ
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button_about_ctrl_Click(object sender, EventArgs e)
        {
            this.DialogResult = DialogResult.Yes;
            Dispose();
        }

        /// <summary>
        /// <Ver 1.18.0></Ver>
        /// ログイン時に MyClock を起動する
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void checkBox_Startup_CheckedChanged(object sender, EventArgs e)
        {
            SetStartup();
        }

        /// <summary>
        /// <Ver 1.18.4></Ver>
        /// [?]ボタンクリックでMyClockサイトをオープンする
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void About_HelpButtonClicked(object sender, CancelEventArgs e)
        {
            System.Diagnostics.Process.Start(
                "https://egadget2.web.fc2.com/archives/Src_files/MyClock.html");
            Dispose();
        }

        /// <summary>
        /// <Ver 1.18.5.1></Ver>
        /// 時計のアイコンをクリックした時 MyClockの紹介ページをブラウザで開く
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void picture_Icon_Click(object sender, EventArgs e)
        {
            System.Diagnostics.Process.Start("https://egadget2.web.fc2.com/archives/Src_files/MyClock.html");
            Dispose();
        }
    }
}
